// An oil lamp for POVRay 3.0.
// 1998 Antti Tuominen
// Noncommercial distribution allowed as long as the material is unchanged.
// Future Publishing's Amiga Format is allowed to make necessary 
// changes to make this material suitable for publication.

#include "colors.inc"
#include "metals.inc"
#include "glass.inc"

#declare lamp_body_texture =
texture {T_Copper_3C finish {reflection 0.3}}

/******************************************************************
 *                       L A M P   B O D Y                        *
 ******************************************************************/

#declare lamp_shape_block =
lathe {
   quadratic_spline
   10,
   <0, 0>,
   <3.6, 0>,
   <3.4, 0.2>,
   <3.1, 1>,
   <3, 1.5>,
   <1.8, 2>,
   <1.6, 2.5>,
   <1.5, 2.6>,
   <0, 2.6>,
   <0, 0>
   translate <0, -2.6/2, 0>
}

#declare lamp_body_shape =
union {
   object {
      lamp_shape_block
      translate <0, 2.6/2, 0>
   }
   object {
      lamp_shape_block
      rotate x*180
      translate <0, 1.3+2.6, 0>
   }
   object {
      lamp_shape_block
      translate <0, 1.3+2*2.6, 0>
   }
}

#declare lamp_body =
object {
   lamp_body_shape
   texture {lamp_body_texture}
}

#declare airhole_drill =
cylinder {
   <-5, 8.7, 0>
   < 5, 8.7, 0>
   0.1 // 2mm airholes
}

#declare airhole_drill_revolution =
union {
   object {airhole_drill rotate <0, 0/20*360, 0>}
   object {airhole_drill rotate <0, 1/20*360, 0>}
   object {airhole_drill rotate <0, 2/20*360, 0>}
   object {airhole_drill rotate <0, 3/20*360, 0>}
   object {airhole_drill rotate <0, 4/20*360, 0>}
   object {airhole_drill rotate <0, 5/20*360, 0>}
   object {airhole_drill rotate <0, 6/20*360, 0>}
   object {airhole_drill rotate <0, 7/20*360, 0>}
   object {airhole_drill rotate <0, 8/20*360, 0>}
   object {airhole_drill rotate <0, 9/20*360, 0>}
   // airhole_drill cuts also on the other side. That's why only
   // a half of a revolution
}

/******************************************************************
 *                  L A M P   M E C H A N I S M                   *
 ******************************************************************/

#declare lamp_mech_inner_shape =
difference {
   lathe {
      quadratic_spline
      22,
      <0, 8>,
      <1.25, 8>,
      <1.3, 8.4>,
      <1.5, 8.5>,
      <1.5, 9.4>,
      <1.2, 9.7>,
      <0.9, 10>,
      <0.8, 11>,
      <0.5, 11.5>,
      <0, 11.7>,
      
      <0, 11.65>,
      <0.45, 11.5>,
      <0.75, 11>,
      <0.85, 10>,
      <1.15, 9.7>,
      <1.4, 9.4>,
      <1.39, 9.39>,  // that darn spline tries to punch a hole to the sheet metal
      <1.4, 9.38>,
      <1.45, 8.5>,
      <1.25, 8.4>,
      <1.2, 8.05>,
      <0, 8>
   }
   box { // make a slot for the heart
      <-0.22, 10.8, -99>
      < 0.22, 99.0,  99>
   }
   object {airhole_drill_revolution} // bottom layer of holes
   object {airhole_drill_revolution translate <0, 0.25, 0> rotate <0, 0.5/20*360>}
   object {airhole_drill_revolution translate <0, 0.5, 0>} 
}

#declare lamp_mech_pole_shape =
union {
   cylinder { // shaft
      <-0.4, 8.1,  2.25>
      <-0.4, 8.1, -3.6>
      0.075
   }
   cylinder { // knob
      <-0.4, 8.1, -3.5>
      <-0.4, 8.1, -3.4>
      0.5
   }
}

#declare glassholder_cutter =
union {
   cylinder {
      <-50, 0, 0>
      < 50, 0, 0>
      0.15
   }
   intersection {
      plane {z, 0 rotate <11, 0, 0> translate <0, -0.4, 0>}
      plane {-z, 0 rotate <-11, 0, 0> translate <0, -0.4, 0>}
      plane {-y, 0}
   }
   translate <0, 10, 0>
}

#declare lamp_mech_glassholder_shape =
difference {
   lathe {
      quadratic_spline
      11,
      <1.5, 9.4>,
      <1.7, 9.6>,
      <1.5, 9.8>,
      <1.4, 10.4>,
      <1.6, 10.8>,
      
      <1.55, 10.8>,
      <1.35, 10.4>,
      <1.45, 9.8>,
      <1.65, 9.6>,
      <1.45, 9.4>,
     
      <1.5, 9.4>
   }
   object {glassholder_cutter rotate <0, 0/14*360, 0>}   
   object {glassholder_cutter rotate <0, 1/14*360, 0>}   
   object {glassholder_cutter rotate <0, 2/14*360, 0>}   
   object {glassholder_cutter rotate <0, 3/14*360, 0>}   
   object {glassholder_cutter rotate <0, 4/14*360, 0>}   
   object {glassholder_cutter rotate <0, 5/14*360, 0>}   
   object {glassholder_cutter rotate <0, 6/14*360, 0>}   
}

#declare lamp_mechanism_shape =
union {
   object {lamp_mech_inner_shape}
   object {lamp_mech_pole_shape}
   object {lamp_mech_glassholder_shape}
}

#declare lamp_mech_heart =
union {
   cylinder {
      <0, 8,    0>
      <0, 11.1, 0>
      0.7
      scale <0.25, 1, 1> // squeeeze... :-)
      texture {
      	 pigment {color Gray50}
      	 finish {
      	    specular 0.1
      	    metallic
      	    roughness 0.9
      	 }
      }
   }
   cylinder {
      <0, 8,    0>
      <0, 11.3, 0>
      0.7
      scale <0.8*0.25, 1, 0.8*1> // squeeeze some more
      texture {
         pigment {color Gray20}
         normal {bumps 1} // some lame disturbance, nobody's gonna see it
                          // that closely anyway
      }
   }
}

#declare lamp_mechanism =
union {
   object {
      lamp_mechanism_shape
      translate <0, -0.2, 0> // It's slightly in the wrong place, but
                             // I'm too lazy to rewrite all those lathe
                             // coordinates
      texture {lamp_body_texture}
   }
   object {
      lamp_mech_heart
      translate <0, -0.2, 0> // and now everything is misplaced since
                             // this stuff is done relative to 
                             // lamp_mechanism_shape
   }
}

/******************************************************************
 *                L A M P   S H A D E H O L D E R                 *
 ******************************************************************/

#declare holder_arm =
union {
   box {  // lower platform
      <1.6, 0.05, -0.3>
      <2.2, 0.15,  0.3>
   }
   box { // higher platform
      <6.2, 0.05+3, -0.3>
      <6.8, 0.15+3,  0.3>
   }
   difference { // hook to the end
      cylinder {
         <6.8, 3.05+0.25, -0.3>
         <6.8, 3.05+0.25,  0.3>
         0.25
      }
      cylinder {
         <6.8, 3.05+0.25, -1>
         <6.8, 3.05+0.25,  1>
         0.15
      }
      plane {x, 6.8}
   }
   box { // the arm itself
      <0,  -0.05, -0.3>
      <5.03,   0.05,  0.3>
      rotate <0, 0, 36.8>
      translate <2.2, 0.1, 0>
   }
}

#declare lamp_shadeholder_shape =
union {
   difference {
      cylinder {
         <0, -0.05, 0>
         <0,  0.05, 0>
         2.2
      }
      cylinder {
         <0, -1, 0>
         <0,  1, 0>
         1.6
      }
   }
   object {holder_arm}
   object {holder_arm rotate <0, 120, 0>}
   object {holder_arm rotate <0,-120, 0>}
}

#declare nail =
cylinder {
   <1.9, -0.1, 0>
   <1.9,  0.25,0>
   0.13
   texture {T_Silver_3B}
}

#declare lamp_shadeholder_nails =
union {
   object {nail}
   object {nail rotate <0, 120, 0>}
   object {nail rotate <0,-120, 0>}
}

#declare lamp_shadeholder =
union {
   object {
      lamp_shadeholder_shape
      texture {lamp_body_texture}
   }
   object {
      lamp_shadeholder_nails
      texture {T_Silver_3A}
   }
   translate <0, 9.7, 0>
}

/******************************************************************
 *                      L A M P   G L A S S                       *
 ******************************************************************/

#declare lamp_glass_shape_full =
union {
   lathe {
      linear_spline
      5,
      <0, 0>,
      <1.38, 0>,
      <1.5, 1.2>,
      <0, 1.2>,
      <0, 0>
   }
   lathe {
      quadratic_spline
      11,
      <0, 1.2>,
      <1.5, 1.2>,
      <2.4, 1.8>,
      <2.625, 2.5>,
      <2.5, 3.5>,
      <1.95, 4.4>,
      <1.5, 5.5>,
      <1.3, 9>,
      <1.29, 9>,
      <0, 9>,
      <0, 1.2>      
   }
}

#declare lamp_glass_shape = 
difference {
   object {lamp_glass_shape_full}
   object {lamp_glass_shape_full scale <0.95, 1.0000001, 0.95>}
}

#declare upper_lamp_glass =
intersection {
   object {lamp_glass_shape}
   plane {-y, -3.5}
   texture {
      pigment {color rgbf <1.0, 1.0, 1.0, 0.6>}
      finish {
      	 ambient 0.2
      	 diffuse 0.6
      	 crand 0.04
         specular 0.5
         roughness 0.05
      	 refraction 1
      	 ior 1.45
      	 phong 0.3
      	 phong_size 400
      }
   }
}

#declare lower_lamp_glass =
intersection {
   object {lamp_glass_shape}
   plane {y, 3.5}
   texture {
      T_Glass3
      finish {
         diffuse 0.6
      }
   }
}

#declare lamp_glass =
union {
   object {upper_lamp_glass}
   object {lower_lamp_glass}
   scale 0.9 // fitting right
}

/******************************************************************
 *                        L A M P    S H A D E                    *
 ******************************************************************/

#declare lamp_shade =
lathe {
   linear_spline
   25,
   <6.25, 0>,
   <5.5, 0.7>,
   <5.25, 0.7>,
   <5.25, 0.85>,
   <4.6, 1.4>,
   <4.35, 1.4>,
   <4.35, 1.6>,
   <3.7, 2.2>,
   <3.45, 2.2>,
   <3.45, 2.4>,
   <2.6, 3>,
   <1.5, 3>,
    
   <1.5, 2.95>,
   <2.6, 2.95>,
   <3.4, 2.3>,
   <3.4, 2.15>,
   <3.65, 2.15>,
   <4.3, 1.55>,
   <4.3, 1.35>,
   <4.55, 1.35>,
   <5.2, 0.8>,
   <5.2, 0.65>,
   <5.45, 0.65>,
   <6.2, 0>,
   
   <6.25, 0>
   texture {lamp_body_texture}
   scale <1.085, 1, 1.085> // fits the holder better
}

/******************************************************************
 *                         L A M P   F L A M E                    *
 ******************************************************************/

#declare flame_pigment =
pigment {
   wood
   color_map {
      [0    rgb <1, 0, 0  >]
      [0.75 rgb <1, 1, 0  >]
      [1    rgb <1, 1, 0.8>]
   } 
   turbulence 0.005
   translate <0, 0.1, 0>
   scale 4
   rotate y*90
}

#declare lamp_flame =
blob {
   threshold 0.1
   sphere {<0, 1, 0>, 1, 1
      pigment {flame_pigment}
      finish {ambient 0.8 diffuse 0.6}
   }
   sphere {<0, 1.5, 0>, 0.75, 1
      pigment {flame_pigment}
      finish {ambient 0.8 diffuse 0.6}
   }
   sphere {<0, 1.75, 0>, 0.5, 1
      pigment {flame_pigment}
      finish {ambient 1 diffuse 0.6}
   }
   scale <0.3, 2, 0.8>
}

/******************************************************************
 *                              L A M P                           *
 ******************************************************************/

#declare lamp =
union {
   object {lamp_body}
   object {
      lamp_mechanism
      translate <0, 0, 0>
   }
   object {
      lamp_shadeholder
      translate <0, 0, 0>
   }
   object {
      lamp_shade
      translate <0, 12.88, 0>      
   }
   object {
      lamp_glass
      translate <0, 9.5, 0>
   }
}

#declare lamp_rotate = <0, 45, 0>
#declare lamp_translate = <0, 0, 0>

light_source {
   <0, 13, 0> color rgb <1.0, 1.0, 0.6>
   looks_like {lamp_flame translate <0, -3, 0>}
   rotate lamp_rotate
   translate lamp_translate
}

// for test purposes
object {lamp
   rotate lamp_rotate
   translate lamp_translate
}
